extern crate backend;

use actix_web::{get, post, App, HttpResponse, HttpServer, Responder};
use actix_web::http::header::ContentType;
use serde_json::{json, Value};

macro_rules! get_value_str {
    ($i:expr) => {
        match $i {
            Value::String(s) => s,
            _ => String::from("")
        }
    }
}

macro_rules! get_value_int {
    ($i:expr) => {
        match $i {
             Value::Number(n) => n.as_i64().unwrap(),
             _ => 0
        }
    }
}

#[post("/new_user")]
async fn createuser(req_body: String) -> impl Responder {
    use backend::UserCreationStatus;
    use backend::UserSelectionStatus;

    let connection = &mut backend::create_connection();

    let v: Value = match serde_json::from_str(&req_body) {
        Ok(v) => v,
        Err(_) => {
            return HttpResponse::BadRequest().body("");
        }
    };

    let email = String::from(v["email"].as_str().unwrap());
    let username = String::from(v["username"].as_str().unwrap());
    let password = String::from(v["password"].as_str().unwrap());

    let status: UserCreationStatus = backend::add_user(
        username,
        email.clone(),
        password,
        connection
    );

    let id = match backend::get_id_from_email(email, connection) {
        UserSelectionStatus::Success(v) => v,
        _ => -1
    };

    let (status, status_code, mut builder) = match status {
        UserCreationStatus::Success => ("Success", 200, HttpResponse::Ok()),
        UserCreationStatus::UserExists => ("User Already Exists", 400, HttpResponse::Conflict()),
        UserCreationStatus::DatabaseError(_) => ("Database Error", 500, HttpResponse::InternalServerError()),
        UserCreationStatus::QueryError(_) => ("Query Error", 500, HttpResponse::InternalServerError())
    };

    builder.insert_header(ContentType::json());

    builder.body(json!({
        "status": status,
        "status_code": status_code,
        "id": id
    }).to_string())
}

#[get("/check_password")]
async fn check_password(req_body: String) -> impl Responder {
    use backend::UserSelectionStatus;

    let connection = &mut backend::create_connection();

    let v: Value = match serde_json::from_str(&req_body) {
        Ok(v) => v,
        Err(_) => {
            return HttpResponse::BadRequest().body("");
        }
    };
    
    let id = get_value_int!(&v["id"]);
    let pass = get_value_str!(v["password"].clone());

    match backend::check_password(id, pass, connection) {
        UserSelectionStatus::Success(b) => HttpResponse::Ok().insert_header(ContentType::json()).body(json!({
            "correct": b,
            "status": "Success",
            "status_code": 200
        }).to_string()),
        UserSelectionStatus::UserDoesntExist => HttpResponse::ExpectationFailed().insert_header(ContentType::json()).body(json!({
            "correct": false,
            "status": "User Doesn't Exist",
            "status_code": 400
        }).to_string()),
        UserSelectionStatus::DatabaseError(_) => HttpResponse::InternalServerError().insert_header(ContentType::json()).body(json!({
            "correct": false,
            "status": "Database Error",
            "status_code": 500
        }).to_string()),
        UserSelectionStatus::QueryError(_) => HttpResponse::InternalServerError().insert_header(ContentType::json()).body(json!({
            "correct": false,
            "status": "Query Error",
            "status_code": 500
        }).to_string())
    }
}

#[actix_web::main]
async fn main() -> std::io::Result<()> {
    HttpServer::new(|| App::new().service(createuser).service(check_password))
        .bind(("127.0.0.1", 8080))?
        .run()
        .await
}
